#include "burnint.h"
// Out Run

static unsigned char bGear=0,bLastGear=0;
static unsigned char DrvButton[8]={0,0,0,0,1,0,0,0}; // Start in low gear
static unsigned char DrvInput[6]={0x00,0x00,0x05,0x80,0x80,0x80};
static unsigned char DrvReset=0;

static int DrvZipName(char **pszName,unsigned int i)
{
  if (i>0) return 1;
  if (pszName!=NULL) { *pszName="outrun.zip"; return 0; }
  return 1;
}

// Rom information
static struct { char szName[16]; int nLen; unsigned int nCrc; int nType; } DrvRomDesc[]=
{
  {"10380a",0x10000,0x434fadbc, 0x10}, //  0 cpu 1 code (even)
  {"10381a",0x10000,0xbe8c412b, 0x10}, //  1
  {"10382a",0x10000,0x1ddcc04e, 0x10}, //  2            (odd)
  {"10383a",0x10000,0xdcc586e7, 0x10}, //  3
  {"10327a",0x10000,0xe28a5baf, 0x10}, //  4 cpu 2 code (even)
  {"10328a",0x10000,0xd5ec5e5d, 0x10}, //  5
  {"10329a",0x10000,0xda131c81, 0x10}, //  6            (odd)
  {"10330a",0x10000,0xba9ec82a, 0x10}, //  7
  {"10268" ,0x08000,0x95344b04,    1}, //  8 tile data
  {"10267" ,0x08000,0xa85bb823,    1}, //  9
  {"10266" ,0x08000,0x9f6f1a74,    1}, // 10
  {"10232" ,0x08000,0x776ba1eb,    1}, // 11
  {"10231" ,0x08000,0x8908bcbf,    1}, // 12
  {"10230" ,0x08000,0x686f5e50,    1}, // 13
  {"10377" ,0x20000,0xc86daecb,    1}, // 14 sprite data
  {"10375" ,0x20000,0x62a472bd,    1}, // 15
  {"10373" ,0x20000,0x339f8e64,    1}, // 16
  {"10371" ,0x20000,0x0a1c98de,    1}, // 17
  {"10378" ,0x20000,0x544068fd,    1}, // 18
  {"10376" ,0x20000,0x8337ace7,    1}, // 19
  {"10374" ,0x20000,0x22744340,    1}, // 20
  {"10372" ,0x20000,0x1640ad1f,    1}, // 21
  {"10185" ,0x08000,0x22794426,    1}  // 22 ground data
};

// Return 0 if a rom number is defined, and (if pri!=NULL) information about each rom
static int DrvRomInfo(struct BurnRomInfo *pri,unsigned int i)
{
  if (i>=sizeof(DrvRomDesc)/sizeof(DrvRomDesc[0])) return 1; // Check i is in range
  // Return information about the rom
  if (pri!=NULL)
  {
    pri->nLen =DrvRomDesc[i].nLen;
    pri->nCrc =DrvRomDesc[i].nCrc;
    pri->nType=DrvRomDesc[i].nType;
  }
  return 0;
}

// Return possible names for rom number 'i'
static int DrvRomName(char **pszName,unsigned int i,int nAka)
{
  if (i>=sizeof(DrvRomDesc)/sizeof(DrvRomDesc[0])) return 1; // Check i is in range
  if (nAka!=0) return 1; // Roms known by one name
  *pszName=DrvRomDesc[i].szName;
  return 0;
}

static struct BurnInputInfo DrvInputList[]=
{
  {"Coin 1"     ,0, DrvButton+6, "p1 coin"},
  {"Start"      ,0, DrvButton+3, "p1 start"},
  {"Left/Right" ,1, DrvInput+3 , "p1 x-axis"},
  {"Accel"      ,1, DrvInput+4 , "p1 fire 1"},
  {"Change Gear",0, &bGear,      "p1 fire 2"},
  {"Brake"      ,1, DrvInput+5 , "p1 fire 3"},

  {"Coin 2"     ,0, DrvButton+7, "p2 coin"},

  {"Reset"      ,0, &DrvReset,   "reset"},
  {"Diagnostic" ,0, DrvButton+1, "diag"},
  {"Dip A"      ,2, DrvInput+1 , "dip"},
  {"Dip B"      ,2, DrvInput+2 , "dip"},
};

// Return 0 if a input number is defined, and (if pii!=NULL) information about each input
static int DrvInputInfo(struct BurnInputInfo *pii,unsigned int i)
{
  if (i>=sizeof(DrvInputList)/sizeof(DrvInputList[0])) return 1;
  if (pii!=NULL) *pii=DrvInputList[i];
  return 0;
}

static int OutResetCallback()
{
  // Reset instruction on cpu 1
  int nLast; nLast=nSekActive;
  SekClose();
  SekOpen(1); SekReset(); SekClose(); // Reset cpu 2
  SekOpen(nLast);
  return 0;
}

static unsigned char *Mem=NULL,*MemEnd=NULL;
static unsigned char *RamStart,*RamEnd;
static unsigned char *Rom1, *Rom2;
static unsigned char *Ram06,*Ram10,*Ram11,*Ram12,*Ram13,*Ram26,*Ram28,*Ram29;
static unsigned int  *HighPal; static int nColCount=0x1000;
static unsigned char *Tile;    static int nTileLen=0x40000;    // Tile data
static unsigned char *SprData; static int nSprDataLen=0x100000; // Sprite data

// This routine is called first to determine how much memory is needed (MemEnd-(unsigned char *)0),
// and then afterwards to set up all the pointers
static int MemIndex()
{
  unsigned char *Next; Next=Mem;
  Rom1 =Next; Next+=0x040000; // [118/133 117/132]
  Rom2 =Next; Next+=0x040000; // [ 58/ 76  57/ 75]
  Tile =Next; Next+=nTileLen;      // tile data
  SprData=Next; Next+=nSprDataLen;   // sprite data
  RamStart=Next;
  Ram06=Next; Next+=0x008000; 
  Ram10=Next; Next+=0x010000; // tile    [63/65]
  Ram11=Next; Next+=0x001000; // text    [64/62]
  Ram12=Next; Next+=0x002000; // palette [95/92]
  Ram13=Next; Next+=0x000800; // sprites
  Ram28=Next; Next+=0x001000; // Ground data
  Ram29=Next; Next+=0x000004; // Road on/off flags
  Ram26=Next; Next+=0x008000; // 54/72
  RamEnd=Next;
  HighPal=(unsigned int *)Next; Next+=nColCount*sizeof(unsigned int);
  MemEnd=Next;
  return 0;
}

// Scan ram
int DrvScan(int nAction,int *pnMin)
{
  struct BurnArea ba;

  if (nAction&4) // Scan volatile ram
  {
    if (pnMin!=NULL) *pnMin=0x00870c; // Return minimum compatible version

    memset(&ba,0,sizeof(ba));
    ba.Data=RamStart;
    ba.nLen=RamEnd-RamStart;
    ba.szName="All Ram";
    BurnAcb(&ba);

    SCAN_VAR(DrvButton[4]) // Scan gear state

    SekScan(nAction&3); // scan 68000 states

  }

  return 0;
}

static int LoadRoms()
{
  int nRet=0; int i=0;
  // Load program roms and byteswap
  nRet=BsysLoadCode(Rom1,0,4); if (nRet!=0) return 1;
  nRet=BsysLoadCode(Rom2,4,4); if (nRet!=0) return 1;

  // Load tile data (three bitfields)
  BsysLoadTiles(Tile+0x00000, 8);
  BsysLoadTiles(Tile+0x20000,11);

  // Load the sprite roms and interleve by 4
  for (i=0;i<8;i++) BurnLoadRom(SprData+0x80000*(i>>2)+(i&3),14+i,4);
  BsysZeroSprites(SprData,nSprDataLen); // Change color 15 to color 0
  return 0;
}

static unsigned char Drv1ReadByte(unsigned int a)
{
  if (a==0x140011) return (unsigned char)(~DrvInput[0]);
  if (a==0x140015) return (unsigned char)(~DrvInput[1]);
  if (a==0x140017) return (unsigned char)(~DrvInput[2]);
  return 0;
}

static void Drv1WriteByte(unsigned int a,unsigned char d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x12) BsysPalWriteByte(a,d); // palette write
}

static void Drv1WriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x12) BsysPalWriteWord(a,d); // palette write
}

static int DrvDoReset()
{
  int i=0;
  for (i=0;i<2;i++) { SekOpen(i); SekReset(); SekClose(); } // Reset cpus
  return 0;
}

static int DrvInit()
{
  int nRet=0; int nLen=0;

  // Find out how much memory is needed
  Mem=NULL; MemIndex();
  nLen=MemEnd-(unsigned char *)0;
  Mem=(unsigned char *)malloc(nLen); if (Mem==NULL)  return 1;
  memset(Mem,0,nLen); // blank all memory
  MemIndex(); // Index the allocated memory

  nRet=LoadRoms(); if (nRet!=0) return 1; // Load the roms into memory

  SekInit(2); // Allocate 2 68000s
  SekExt[0].ResetCallback=OutResetCallback; // Get cpu 1 reset requests

  // Map in memory:
  // ----------------- Cpu 1 ------------------------
  SekOpen(0);
  SekMemory(Rom1 ,0x000000,0x03ffff,SM_ROM); // cpu 1 Rom
  SekMemory(Ram06,0x060000,0x067fff,SM_RAM);
  SekMemory(Ram10,0x100000,0x10ffff,SM_RAM);
  SekMemory(Ram11,0x110000,0x110fff,SM_RAM);
  SekMemory(Ram12,0x120000,0x121fff,SM_ROM); // palette Ram (read-only - write goes through handler)
  SekMemory(Ram13,0x130000,0x1307ff,SM_RAM);
  SekMemory(Rom2 ,0x200000,0x23ffff,SM_ROM); // cpu 2 Rom (visible from cpu 1)
  SekMemory(Ram26,0x260000,0x267fff,SM_RAM); // shared ram with cpu 2

  SekExt[0].ReadByte =Drv1ReadByte;
  SekExt[0].WriteWord=Drv1WriteWord;
  SekExt[0].WriteByte=Drv1WriteByte;

  SekClose();
  // ------------------------------------------------

  // ----------------- Cpu 2 ------------------------
  SekOpen(1);
  SekMemory(Rom2 ,0x000000,0x03ffff,SM_ROM); // cpu 2 Rom
  SekMemory(Ram26,0x060000,0x067fff,SM_RAM);
  SekMemory(Ram28,0x080000,0x080fff,SM_RAM);
  SekMemory(Ram29,0x090000,0x090000,SM_RAM);
  SekClose();
  // ------------------------------------------------

  // ----------------- Draw init --------------------
  // Palette
  BsysPal=HighPal; nBsysPalLen=nColCount; // palette (used for all graphics)
  BsysPalSrc=Ram12; BsysPalInit();

  BsysTileData=Tile;  nBsysTileDataLen=nTileLen; nBsysTilePalBase=0; // Tiles
  BsysSprData=SprData; nBsysSprDataLen=nSprDataLen; // Sprite bitmaps in Roms

  GrOutRam =Ram28;
  GrOutFlag=Ram29;
  // ------------------------------------------------

  DrvDoReset(); // Reset machine
  return 0;
}

static int DrvExit()
{
  // ----------------- Draw exit --------------------
  BsysPalExit();
  // ------------------------------------------------

  SekExit(); // Deallocate 68000s
  // Deallocate all used memory
  if (Mem!=NULL) free(Mem);  Mem=NULL;
  return 0;
}

#define OR_FLAG_A (*((short *)(Ram06+ 0x800)))
#define OR_FLAG_B (*((short *)(Ram26+0x3800)))

// Speedup conditions for PC being at the loop point, and whether we should loop
//***   7d66 tst.w   $60800.l                         [4a 79 00 06 08 00 ]
//      7d6c bge     7d66                             [6c f8 ]
#define PC_A1 (pc==0x7d66 || pc==0x7d6c)
#define  L_A1 (OR_FLAG_A>=0)

//***   78f0 cmpi.w  #$1, $60800.l                    [0c 79 00 01 00 06 08 00 ]
//      78f8 beq     78f0                             [67 f6 ]
#define PC_A2 (pc==0x78f0 || pc==0x78f8)
#define  L_A2 (OR_FLAG_A==1)

//***   1182 tst.w   ($3800,A5)                       [4a 6d 38 00 ]  (a5==0x60000)
//      1186 bge     1182                             [6c fa ]
#define PC_B1 (pc==0x1182 || pc==0x1186)
#define  L_B1 (OR_FLAG_B>=0)

//***   1060 cmpi.w  #$1, ($3800,A5)                  [0c 6d 00 01 38 00 ]
//      1066 bge     1060                             [6c f8 ]
#define PC_B2 (pc==0x1060 || pc==0x1066)
#define  L_B2 (OR_FLAG_B>=1)

static INLINE int CheckSleep(int j)
{
  unsigned int pc; pc=SekPc(j);
  // Out Run sleep/wake up:
  if (j==0) { if (PC_A1) return L_A1; else if (PC_A2) return L_A2; }
  else      { if (PC_B1) return L_B1; else if (PC_B2) return L_B2; }
  // End of Outrun sleep/wake up
  return 0;
}

static int DrawSprite(unsigned char *ps)
{
  int nPal;
  bss.nTop=ps[0]; bss.nHeight=ps[11]+1;
  bss.nLeft=((ps[5]<<8)|ps[4])&0x1ff; bss.nLeft-=0xc0;
  bss.nWidth=(ps[5]&0xfe)<<2;
  bss.nClipWidth=bss.nWidth;
  nPal=ps[10]&0x7f; nPal+=0x80; nPal<<=4;
  bss.pPal=BsysPal+nPal;

  // Find the bitmap in the rom data
  bss.nBase=((ps[1]&0x06)<<17) | (ps[3]<<10) | (ps[2]<<2);

  // Find the zoom values
  {
    int nZoomX,nZoomY;
    nZoomX=((ps[7]<<8)|ps[6])&0xfff;
    nZoomY=((ps[9]<<8)|ps[8])&0xfff;
    if (nZoomX==0x32c) nZoomX=0x3c2; // mistake in original machine? !
    if (nZoomY==0) nZoomY=nZoomX;
    bss.nIncX=nZoomX; bss.nIncY=nZoomY;
  }

  bss.bFlipX=0;
  if ((ps[9]&0x40)==0) { bss.nBase+=4-(bss.nWidth>>1); bss.bFlipX=1; }
  bss.bDrawToLeft=((ps[9]&0x20)==0);
  bss.bDrawToTop =0;

  // Now that the sprite is in a standard format, draw it
  bss.nTrans=0x0001; //normal transparent mask
  BsysSprDraw();

  return 0;
}

static int DrvSprDraw()
{
  int i=0; unsigned char *ps=NULL;
  ps=Ram13;
  for (i=0; i<0x80; ps+=0x10,i++)
  {
    if (ps[0]==0xff && ps[1]==0xff) break; // end of sprite list
    if (ps[1]&0x40) continue; // sprite not active
    DrawSprite(ps);
  }
  return 0;
}

static int DrvDraw()
{
  static unsigned Restore[4]; int nPos=0;

  BsysPalUpdate(); // Update the palette
  BurnClearScreen();
  GrOutDraw();
  BsysTilePage(0,0,Ram10); // Draw tiles
  DrvSprDraw();

  // Put text to display high and low gear
  nPos=0xcb2;
  memcpy(Restore,Ram11+nPos,4);
  Ram11[nPos+0]=(DrvInput[0]&0x10) ? 'L' : 'H';
  Ram11[nPos+2]=(DrvInput[0]&0x10) ? 'O' : 'I';
  Ram11[nPos+1]=0x05;
  Ram11[nPos+3]=0x05;

  BsysTilePage(0,0,Ram11); // Draw text
  memcpy(Ram11+nPos,Restore,4);

  return 0;
}

static int DrvFrame()
{
  int nDid[2],nTotal[2],i=0,nInterleve=10;

  if (DrvReset) DrvDoReset(); // Reset machine

  if (bGear && bLastGear==0) DrvButton[4]^=1; // Toggle low/high gear
  bLastGear=bGear;

  // Compile separate buttons into DrvInput[0]
  DrvInput[0]=0; for (i=0;i<8;i++) DrvInput[0]|=(DrvButton[i]&1)<<i;

  // Insert joystick info into ram:
  Ram06[0x0892^1]=DrvInput[3]; // x-axis
  Ram06[0x0893^1]=DrvInput[4]; // y-axis
  Ram06[0x0894^1]=DrvInput[5]; // z-axis

  nTotal[0]=12000000/60; // 12mhz
  nTotal[1]=nTotal[0];
  nDid[0]=nDid[1]=0;

  for (i=0;i<nInterleve;i++)
  {
    int j=0; 
    // Run both cpus
    for (j=0;j<2;j++)
    {
      int bSleep; bSleep=CheckSleep(j);
      if (bSleep) continue; // See if this cpu can sleep

      {
        int nNext; // Next cycle count to run to
        SekOpen(j);
        nNext=(i+1)*nTotal[j]/nInterleve; // find out next cycle count to run to
        nDid[j]+=SekRun(nNext-nDid[j]);  // run cpu
        SekClose();
      }
    }
  }

  // Let cpus run for a bit after the vblank so that the palette is defined
  SekOpen(0); SekInterrupt(4); SekRun(2000); SekClose();
  SekOpen(1); SekInterrupt(4); SekRun(2000); SekClose();

  if (pBurnDraw!=NULL) DrvDraw(); // Draw screen if needed
  return 0;
}

struct BurnDriver BurnDrvOutrun=
{
  {"outrun","Out Run","Preliminary driver"},
  DrvZipName,DrvRomInfo,DrvRomName,DrvInputInfo,
  DrvInit,DrvExit,DrvFrame,DrvScan,NULL,320,224
};
