#include "burnint.h"
// Street Fighter 2 - Driver

static struct BurnInputInfo DrvInputList[]=
{
  {"P1 Coin"       , 0, CpsInp018+0, "p1 coin"},
  {"P1 Start"      , 0, CpsInp018+4, "p1 start"},
  {"P1 Up"         , 0, CpsInp001+3, "p1 up"},
  {"P1 Down"       , 0, CpsInp001+2, "p1 down"},
  {"P1 Left"       , 0, CpsInp001+1, "p1 left"},
  {"P1 Right"      , 0, CpsInp001+0, "p1 right"},
  {"P1 Low Punch"  , 0, CpsInp001+4, "p1 fire 4"},
  {"P1 Mid Punch"  , 0, CpsInp001+5, "p1 fire 5"},
  {"P1 High Punch" , 0, CpsInp001+6, "p1 fire 6"},
  {"P1 Low Kick"   , 0, CpsInp177+0, "p1 fire 1"},
  {"P1 Mid Kick"   , 0, CpsInp177+1, "p1 fire 2"},
  {"P1 High Kick"  , 0, CpsInp177+2, "p1 fire 3"},

  {"P2 Coin"       , 0, CpsInp018+1, "p2 coin"},
  {"P2 Start"      , 0, CpsInp018+5, "p2 start"},
  {"P2 Up"         , 0, CpsInp000+3, "p2 up"},
  {"P2 Down"       , 0, CpsInp000+2, "p2 down"},
  {"P2 Left"       , 0, CpsInp000+1, "p2 left"},
  {"P2 Right"      , 0, CpsInp000+0, "p2 right"},
  {"P2 Low Punch"  , 0, CpsInp000+4, "p2 fire 4"},
  {"P2 Mid Punch"  , 0, CpsInp000+5, "p2 fire 5"},
  {"P2 High Punch" , 0, CpsInp000+6, "p2 fire 6"},
  {"P2 Low Kick"   , 0, CpsInp177+4, "p2 fire 1"},
  {"P2 Mid Kick"   , 0, CpsInp177+5, "p2 fire 2"},
  {"P2 High Kick"  , 0, CpsInp177+6, "p2 fire 3"},

  {"Diagnostic"    , 0, CpsInp018+6, "diag"},
  {"Reset"         , 0, &CpsReset  , "reset"},
  {"Dip A"         , 2, &Cpi01A    , "dip"},
  {"Dip B"         , 2, &Cpi01C    , "dip"},
  {"Dip C"         , 2, &Cpi01E    , "dip"},
};

// Return 0 if a input number is defined, and (if pii!=NULL) information about each input
static int DrvInputInfo(struct BurnInputInfo *pii,unsigned int i)
{
  if (i>=sizeof(DrvInputList)/sizeof(DrvInputList[0])) return 1;
  if (pii!=NULL) *pii=DrvInputList[i];
  return 0;
}

static int DrvScode(unsigned int n)
{
  if (n>=0x01 && n<0x1f) return 2;
  if (n>=0x34 && n<0x35) return 2;
  if (n>=0x79 && n<0x84) return 2;
  if (n>=0x8c && n<0x8d) return 2;

  if (n>=0x8f && n<0xad) return 2;
  if (n>=0xc2 && n<0xc3) return 2;
  return 1;
}

static int DrvInit()
{
  int nRet=0; int i=0;
  Cps=1;
  nCpsRomLen =  0x100000;
  nCpsGfxLen =3*0x200000;
  nCpsZRomLen=  0x010000;
  nCpsAdLen  =2*0x020000;
  nRet=CpsInit(); if (nRet!=0) return 1;

  // Load program roms
  for (i=0;i<8;i++)
  {
    int a; a=(i>>1)*0x40000; a+=(i&1); a^=1;
    nRet=BurnLoadRom(CpsRom+a,i,2); if (nRet!=0) return 1;
  }

  // Load up and interleve each set of 4 roms to make the 16x16 tiles
  for (i=0;i<3;i++) CpsLoadTiles(CpsGfx+i*0x200000,8+i*4);

  nCpsGfxScroll[1]=nCpsGfxScroll[2]=nCpsGfxScroll[3]=0x400000; // Offset to Scroll tiles

  nCpsLcReg=0x54; // Layer control register is at 0x54
  CpsLayEn[1]=0x08; CpsLayEn[2]=0x10; CpsLayEn[3]=0x02; // Layer enable is different

  // Load Z80 Rom
  nRet=BurnLoadRom(CpsZRom,20,1);
  BurnScode=DrvScode;

  // Load ADPCM data
  nRet=BurnLoadRom(CpsAd        ,21,1);
  nRet=BurnLoadRom(CpsAd+0x20000,22,1);

  // Set dip switches
  Cpi01A=0x00;
  Cpi01C=0x03;
  Cpi01E=0x60;

  nPsndIrqPeriod=(60<<10)/250; //OLDTST
  nRet=CpsRunInit();


  if (nRet!=0) return 1;
  // Ready to go
  return 0;
}

static int DrvExit()
{
  CpsRunExit();
  nPsndIrqPeriod=0; //OLDTST
  BurnScode=NULL;
  CpsExit();
  nCpsAdLen=0; nCpsZRomLen=0; nCpsGfxLen=0; nCpsRomLen=0;
  Cps=0;
  return 0;
}

static int DrvFrame()
{
  CpsFsStopOpposite(); // stop opposite directions
  // Carry on to CPS frame
  CpsFrame();
  return 0;
}

// Street Fighter 2 CE ----------------------------------
static int Sf2ZipName(char **pszName,unsigned int i)
{
  if (i==0) { if (pszName!=NULL) *pszName="sf2.zip"; return 0; }
  return 1;
}

static struct StdRomInfo Sf2RomDesc[]=
{
// 0
  {"sf2e_30b.rom",0x20000,0x57bd7051, 0x10}, //  0 68000 code
  {"sf2e_37b.rom",0x20000,0x62691cdd, 0x10},
  {"sf2e_31b.rom",0x20000,0xa673143d, 0x10},
  {"sf2e_38b.rom",0x20000,0x4c2ccef7, 0x10},
  {"sf2_28b.rom" ,0x20000,0x4009955e, 0x10},
  {"sf2_35b.rom" ,0x20000,0x8c1f3994, 0x10},
  {"sf2_29b.rom" ,0x20000,0xbb4af315, 0x10},
  {"sf2_36b.rom" ,0x20000,0xc02a13eb, 0x10},

// 8
  {"sf2gfx02.rom",0x80000,0x22c9cc8e,    1},
  {"sf2gfx04.rom",0x80000,0x57213be8,    1},
  {"sf2gfx01.rom",0x80000,0xba529b4f,    1},
  {"sf2gfx03.rom",0x80000,0x4b1b33a8,    1},
  {"sf2gfx11.rom",0x80000,0x2c7e2229,    1},
  {"sf2gfx13.rom",0x80000,0xb5548f17,    1},
  {"sf2gfx10.rom",0x80000,0x14b84312,    1},
  {"sf2gfx12.rom",0x80000,0x5e9cd89a,    1},
  {"sf2gfx21.rom",0x80000,0x994bfa58,    1},
  {"sf2gfx23.rom",0x80000,0x3e66ad9d,    1},
  {"sf2gfx20.rom",0x80000,0xc1befaa8,    1},
  {"sf2gfx22.rom",0x80000,0x0627c831,    1},

// 20
// z80 program
  {"sf2_09.rom"  ,0x10000,0xa4823a1b,    2},
// adpcm samples
  {"sf2_18.rom"  ,0x20000,0x7f162009,    2},
  {"sf2_19.rom"  ,0x20000,0xbeade53f,    2},
};

// Make The RomInfo/Name functions for the game
STD_ROM_PICK(Sf2) STD_ROM_FN(Sf2)

struct BurnDriver BurnDrvCpsSf2=
{
  {"sf2","Street Fighter 2",""},
  Sf2ZipName,Sf2RomInfo,Sf2RomName,DrvInputInfo,
  DrvInit,DrvExit,DrvFrame,CpsAreaScan,
  &CpsRecalcPal,384,224
};
