#include "burnint.h"
// Galaxy Force HW - Run module

static unsigned char *RomM,*RomX,*RomY;
static unsigned char *LineData; static int nLineDataLen=0x80000;  // Line Sprite bitmap data
static unsigned char * SprData; static int  nSprDataLen=0; // Normal Sprite bitmap data
static unsigned char *RamStart=NULL,*RamEnd=NULL;
static unsigned char *RamMFF,*RamXFF,*RamYFF;
static unsigned char *Ram0C,*RamM18,*RamM19,*RamX18;
static unsigned int  *HighPal; static int nColCount=0x4000;
static unsigned char *Mem=NULL,*MemEnd=NULL;
static unsigned char *DrvPcm;   static int  nDrvPcmLen=0;

unsigned char GalGame=0; // 0x10=GF2, 0x20=PD, 0x30=RC, 0x40=G-Loc

unsigned char GalButton[8]={0,0,0,0,0,0,0,0};
unsigned char GalAnalog[GAL_ANALOG_COUNT]={0x80,0x80,0x80,0x80,0x80,0x80};
static unsigned char AnaVal[GAL_ANALOG_COUNT]; // Converted analog values
unsigned char bGalGear=0;
unsigned char GalDip[2]={0,0};
unsigned char GalReset=0;

static unsigned char GalInput=0;
static unsigned char bLastGear=0;

// This routine is called first to determine how much memory is needed (MemEnd-(unsigned char *)0),
// and then to set up all the pointers
static int MemIndex()
{
  unsigned char *Next; Next=Mem;
  RomM    =Next;  Next+=0x040000; // cpu M program
  RomX    =Next;  Next+=0x040000; // cpu X program
  RomY    =Next;  Next+=0x080000; // cpu Y program
  LineData=Next;  Next+=nLineDataLen; // Line sprite data
   SprData=Next;  Next+= nSprDataLen; // Normal sprite data
  RamStart=Next;
  RamMFF  =Next;  Next+=0x010000;   
  RamXFF  =Next;  Next+=0x010000;   
  RamYFF  =Next;  Next+=0x010000;   
  Ram0C   =Next;  Next+=0x010000; // shared?
  RamM18  =Next;  Next+=0x010000; // video ic27-30
  RamM19  =Next;  Next+=0x010000; // video ic104/5 (palette?)
  RamX18  =Next;  Next+=0x010000; // video ic3/4
  AftZ80  =Next;  Next+=0x010000; // z80 memory
  RamEnd=Next;
  DrvPcm  =Next;  Next+=nDrvPcmLen; // pcm data
  HighPal=(unsigned int *)Next; Next+=nColCount*sizeof(unsigned int);
  MemEnd  =Next;
  return 0;
}

static int LoadRoms()
{
  int nRet=0,i=0;
  // Load program roms and byteswap
  nRet=BsysLoadCode(RomM,0,2); if (nRet!=0) return 1;
  nRet=BsysLoadCode(RomX,2,2); if (nRet!=0) return 1;
  nRet=BsysLoadCode(RomY,4,4); if (nRet!=0) return 1;

  // Load line sprite data
  if ((GalGame&0xf0)==0x30)
  {
    // Two 256k roms
    nRet=BurnLoadRom(LineData+0x00000, 8,2);
    nRet=BurnLoadRom(LineData+0x00001, 9,2);
  }
  else
  {
    // Four 128k roms
    nRet=BurnLoadRom(LineData+0x00000, 8,2);
    nRet=BurnLoadRom(LineData+0x00001, 9,2);
    nRet=BurnLoadRom(LineData+0x40000,10,2);
    nRet=BurnLoadRom(LineData+0x40001,11,2);
  }
  BsysZeroSprites(LineData,nLineDataLen); // Change color 15 to color 0

  // Load the sprite roms and interleve by 8
  if ((GalGame&0xf0)==0x30) // Rail Chase
  {
    // 24 x 512k roms
    for (i=0;i<24;i++)
    {
      BurnLoadRom(SprData+0x400000*(i>>3)+(i&7),12+i,8);
    }
  }
  else if ((GalGame&0xf0)==0x40) // G-Loc
  {
    // 32 x 512k roms
    for (i=0;i<32;i++)
    {
      BurnLoadRom(SprData+0x400000*(i>>3)+(i&7),12+i,8);
    }
  }
  else
  {
    // 32 x 128k roms
    for (i=0;i<32;i++)
    {
      BurnLoadRom(SprData+0x100000*(i>>3)+(i&7),12+i,8);
    }
  }
  BsysZeroSprites(SprData,nSprDataLen); // Change color 15 to color 0

  // Load the z80 Rom
  BurnLoadRom(AftZ80,44,1);

  // Load PCM data
  memset(DrvPcm,0x80,nDrvPcmLen); // fill sound with silence

  if ((GalGame&0xf0)==0x30 || (GalGame&0xf0)==0x40) // Rail Chase or G-Loc
  {
    // Rail Chase
    BurnLoadRom(DrvPcm+0x000000,47,1);
    BurnLoadRom(DrvPcm+0x080000,46,1);
    BurnLoadRom(DrvPcm+0x100000,45,1);
  }
  else
  {
    BurnLoadRom(DrvPcm+0x000000,45,1);
    BurnLoadRom(DrvPcm+0x080000,46,1);
    BurnLoadRom(DrvPcm+0x0a0000,47,1);
  }
  return 0;
}

static void DrvSndCode(unsigned char nCode)
{
  // See if the code should be blocked
  if (BurnScodePass(nCode)==0) return;

  nAftSoundCode=nCode;
  ZetNmi();
  
  ZetRun(300); // run the z80 to accept the code
}

static int DrvSndGetBank(int nReg86)
{
  int b;
  b=(nReg86>>3)&0x1f;

  if ((GalGame&0xf0)==0x30)
  {
    // Rail Chase
    if (b==7) return 0; // credit sound is wrong
    // Before plane goes over may be wrong?
    if (b< 0x18) return b;
    return -1;
  }

  if (b==16) b=10;
  if (b==17) b=11;
  return b;
}

// Input at 100000 in all cpus
static INLINE unsigned char GalInpRead(unsigned int a)
{
  a&=0xffff;
  if (a==0x3) return (unsigned char)~GalInput;
  if (a==0xb) return (unsigned char)~GalDip[1];
  if (a==0xd) return (unsigned char)~GalDip[0];
  return 0xff;
}

static INLINE void GalInpWrite(unsigned int a,unsigned char d)
{
  (void)d;
  a&=0xffff;
}

// Cpu M -------------------------------
static unsigned char GalMReadByte(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcReadByte(0,a); // math chip
  if (ab==0x10) return GalInpRead(a); // Input
  return 0;
}

static void GalMWriteByte(unsigned int a,unsigned char d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x10) { GalInpWrite(a,d); return; } // Input
  if (ab==0x19) { BsysPalWriteByte(a,d); return; } // palette write
}

static unsigned short GalMReadWord(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcRead(0,a); // math chip
  SEK_DEF_READ_WORD(a)
}

static void GalMWriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) { CalcWrite(0,a,d); return; } // math chip
  if (ab==0x19) { BsysPalWriteWord(a,d); return; } // palette write
  SEK_DEF_WRITE_WORD(a,d)
}
// Cpu X -------------------------------
static unsigned char GalXReadByte(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcReadByte(1,a); // math chip
  if (ab==0x10) return GalInpRead(a);
  return 0;
}

static void GalXWriteByte(unsigned int a,unsigned char d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x10) { GalInpWrite(a,d); return; }
}

static unsigned short GalXReadWord(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcRead(1,a); // math chip
  SEK_DEF_READ_WORD(a)
}

static void GalXWriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) { CalcWrite(1,a,d); return; } // math chip
  SEK_DEF_WRITE_WORD(a,d)
}

// Cpu Y -------------------------------
static unsigned char GalYReadByte(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcReadByte(2,a); // math chip
  if (ab==0x10) return GalInpRead(a);
  return 0;
}

static void GalYWriteByte(unsigned int a,unsigned char d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x10) { GalInpWrite(a,d); return; }
  if (a==0x82001) { DrvSndCode(d); return; }
}

static unsigned short GalYReadWord(unsigned int a)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) return CalcRead(2,a); // math chip
  SEK_DEF_READ_WORD(a)
}

static void GalYWriteWord(unsigned int a,unsigned short d)
{
  int ab; ab=(a>>16)&0xff;
  if (ab==0x08) { CalcWrite(2,a,d); return; } // math chip
  SEK_DEF_WRITE_WORD(a,d)
}

static int DrvReset()
{
  int i=0;
  for (i=0;i<3;i++) { SekOpen(i); SekReset(); SekClose(); } // Reset cpus
  ZetReset(); // reset z80
  return 0;
}

// ----------------------------------------
int GalInit()
{
  int nRet=0; int nLen=0;

  if ((GalGame&0xf0)==0x30) // Rail Chase
  {
    nSprDataLen=0xc00000; // 24x512k
    nDrvPcmLen=0x180000;
  }
  else if ((GalGame&0xf0)==0x40) // G-Loc
  {
    nSprDataLen=0x1000000; // 32x512k
    nDrvPcmLen=0x180000;
  }
  else
  {
    nSprDataLen=0x400000; // 32x128k
    nDrvPcmLen=0xc0000;
  }

  // Find out how much memory is needed
  Mem=NULL; MemIndex();
  nLen=MemEnd-(unsigned char *)0;
  Mem=(unsigned char *)malloc(nLen); if (Mem==NULL)  return 1;
  memset(Mem,0,nLen); // blank all memory
  MemIndex(); // Index the allocated memory

  nRet=LoadRoms(); if (nRet!=0) return 1; // Load the roms into memory

  SekInit(3); // Allocate 3 68000s - M,X,Y

  // Map in memory:
  // ----------------- Cpu M (0) ------------------------
  SekOpen(0);
  SekMemory(RomM  ,0x000000,0x03FFFF,SM_ROM); // Cpu M Rom
  SekMemory(Ram0C ,0x0C0000,0x0CFFFF,SM_RAM);
  SekMemory(RamM18,0x180000,0x18FFFF,SM_RAM);
  SekMemory(RamM19,0x190000,0x197FFF,SM_ROM); // Read-only: write goes through palette handler
  SekMemory(RamMFF,0xFF0000,0xFFFFFF,SM_RAM);
  SekExt[0].ReadByte =GalMReadByte;
  SekExt[0].WriteByte=GalMWriteByte;
  SekExt[0].ReadWord =GalMReadWord;
  SekExt[0].WriteWord=GalMWriteWord;
  SekClose();
  // ----------------- Cpu X (1) ------------------------
  SekOpen(1);
  SekMemory(RomX  ,0x000000,0x03FFFF,SM_ROM); // Cpu X Rom
  SekMemory(Ram0C ,0x0C0000,0x0CFFFF,SM_RAM);
  SekMemory(RamX18,0x180000,0x18FFFF,SM_RAM);
  SekMemory(RamXFF,0xFF0000,0xFFFFFF,SM_RAM);
  SekExt[1].ReadByte =GalXReadByte;
  SekExt[1].WriteByte=GalXWriteByte;
  SekExt[1].ReadWord =GalXReadWord;
  SekExt[1].WriteWord=GalXWriteWord;
  SekClose();
  // ----------------- Cpu Y (2) ---------------------
  SekOpen(2);
  SekMemory(RomY  ,0x000000,0x07FFFF,SM_ROM); // Cpu Y Rom
  SekMemory(Ram0C ,0x0C0000,0x0CFFFF,SM_RAM);
  SekMemory(RamYFF,0xFF0000,0xFFFFFF,SM_RAM);
  SekExt[2].ReadByte =GalYReadByte;
  SekExt[2].WriteByte=GalYWriteByte;
  SekExt[2].ReadWord =GalYReadWord;
  SekExt[2].WriteWord=GalYWriteWord;
  SekClose();
  // ------------------------------------------------

  AftZInit(); // z80 init
  PcmRom=DrvPcm; nPcmBankCount=nDrvPcmLen>>16; nPcmBankSize=16; // bank length = 1<<16 = 0x10000
  PcmGetBank=DrvSndGetBank;
  AftSoundInit(); // Init sound emulation
  
  // ----------------- Draw init --------------------
  BsysPal=HighPal; nBsysPalLen=nColCount; // palette (used for all graphics)
  BsysPalSrc=RamM19; BsysPalInit();

  GalLineData=LineData; nGalLineDataLen=nLineDataLen; // Line sprite bitmaps in Rom
  GalLineSrc=RamM18+0x8000; // Line sprite tables in Cpu M Video ram

  GalSprData=SprData; nGalSprDataLen=nSprDataLen; // Normal sprite bitmaps in Rom
  GalSprSrc=RamX18+0x0000; // Normal sprite tables in Cpu X Video ram
  // ------------------------------------------------

  DrvReset(); // Reset machine

  return 0;
}

int GalExit()
{
  // ----------------- Draw exit --------------------
  BsysPalExit();
  // ------------------------------------------------
  AftSoundExit(); // Exit sound emulation
  PcmRom=NULL; nPcmBankCount=0; nPcmBankSize=0;
  AftZExit(); // z80 exit

  SekExit(); // Deallocate 68000s
  // Deallocate all used memory
  if (Mem!=NULL) free(Mem);  Mem=NULL;

  nDrvPcmLen=0; // Blank pcm data length
  nSprDataLen=0; // Blank sprite data length
  GalGame=0; // blank game
  return 0;
}

static INLINE int CheckSleep(int j)
{
// Galaxy Force loop points
#define PC_GF_M  (pc==0x1406e || pc==0x14072)
#define  L_GF_M  (((char)RamMFF[0xf000^1])>=0)
#define PC_GF_X1 (pc== 0xa0c4 || pc== 0xa0c8)
#define  L_GF_X1 (((char)RamXFF[0xf200^1])>=0)
#define PC_GF_X2 (pc>= 0x9fc4 || pc<= 0x9fd4)
#define  L_GF_X2 (0) //OLDTST (((char)Ram0C [0x000e^1])>=0)
#define PC_GF_Y  (pc== 0x1f70 || pc== 0x1f74)
#define  L_GF_Y  (((char)RamYFF[0xf000^1])>=0)

  if (GalGame==0x10)
  {
    unsigned int pc; pc=SekPc(j);
    // Galaxy Force II sleep/wake up:
         if (j==0) { if (PC_GF_M)  return L_GF_M; }
    else if (j==1) { if (PC_GF_X1) return L_GF_X1; else if (PC_GF_X2) return L_GF_X2; }
    else           { if (PC_GF_Y)  return L_GF_Y; }
    // End of Galaxy Force II sleep/wake up
  }

// Power Drift loop points
#define PC_PD_M  (pc==0x1188e || pc==0x11892)
#define  L_PD_M  (RamMFF[0xf9f0^1])
#define PC_PD_X  (pc==0xdb90 || pc==0xdb96)
#define  L_PD_X  (Ram0C[0xff0c^1])
#define PC_PD_Y  (pc==0x33084 || pc==0x3308a)
#define  L_PD_Y  (((char)Ram0C[0xff0e^1])>=0)

  if (GalGame==0x20)
  {
    unsigned int pc; pc=SekPc(j);
    // Power Drift sleep/wake up:
         if (j==0) { if (PC_PD_M) return L_PD_M; }
    else if (j==1) { if (PC_PD_X) return L_PD_X; }
    else           { if (PC_PD_Y) return L_PD_Y; }
    // End of Power Drift sleep/wake up
  }
  return 0;
}

static INLINE void InputPrepare()
{
  switch (GalGame&0xf0)
  {
    case 0x10: // Galaxy Force
      AnaVal[1]=(unsigned char)(0x100-AnaVal[1]);
    break;
    case 0x20: // Power Drift
      AnaVal[5]=(unsigned char)(0x30+(AnaVal[5]*0xa/0x10));
    break;
    case 0x40: // G-Loc
      AnaVal[3]=(unsigned char)(0x100-AnaVal[3]);
    break;
  }
}

static INLINE void InputMemory()
{
  unsigned char *pm=NULL; int i=0;
  switch (GalGame&0xf0)
  {
    case 0x20: case 0x40: pm=Ram0C+0xff22; break; // Power Drift/G-Loc
    default:   pm=Ram0C+0x00ca; break; // Galaxy Force/Rail Chase
  }
  if (pm==NULL) return;
  for (i=0;i<GAL_ANALOG_COUNT;i++) pm[i^1]=AnaVal[i];
}

static INLINE int GearDisplay()
{
  // Display the current gear on the screen
  static unsigned char ps[16]={0x68,0x70,0xef,0,0x10,0,0,0,0x2b,0x03,0,0,0,0,0,0};
  // Setup up the BsysSpr for the Line sprites:
  BsysSprData=GalLineData; nBsysSprDataLen=nGalLineDataLen;
  ps[0]=216; ps[1]=216+8;  ps[3]=0x00; ps[2]=0xb8; // Y,X
  if (GalButton[5]) { ps[6]=0xa0; ps[7]=0xad; } // 75b60  low gear
  else              { ps[6]=0x20; ps[7]=0xae; } // 75c60 high gear
  GalLineDrawSprite(0,ps);
  BsysSprData=NULL; nBsysSprDataLen=0;
  return 0;
} 

// Draw graphics
static int DrvDraw()
{
  int nSkyOff=0;
  BsysPalUpdate(); // Update the palette

  nSkyOff=0x88;
  if ((GalGame&0xf0)==0x20) nSkyOff=0x30; // Power Drift
  if ((GalGame&0xf0)==0x40) nSkyOff=0x30; // G-Loc

  {
    // sky
    unsigned char *pl=NULL; int y=0;
    for (pl=pBurnDraw,y=0; y<224; pl+=nBurnPitch,y++)
    { unsigned int c; c=HighPal[nSkyOff+y]; BurnFillLine(pl,c); }
  }

  GalSprDraw();
  GalLineDraw();
  if ((GalGame&0xf0)==0x20) GearDisplay(); // show power drift gear state

  return 0;
}

// Do one frame
int GalFrame()
{
  int nDone[4],nTotal[4],i=0,nInterleve=100;
  if (GalReset) DrvReset(); // Reset machine

  if ((GalGame&0xf0)==0x20)
  {
    if (bGalGear && bLastGear==0) GalButton[5]^=1; // Toggle low/high gear
    bLastGear=bGalGear;
  }

  // Compile separate buttons into GalInput
  GalInput=0; for (i=0;i<8;i++) GalInput|=(GalButton[i]&1)<<i;

  // Make Analog values to return via address 100000
  for (i=0;i<GAL_ANALOG_COUNT;i++) AnaVal[i]=GalAnalog[i];

  InputPrepare();

  nTotal[0]=nTotal[1]=nTotal[2]=12000000/60; // 12mhz 68000
  nTotal[3]=4000000/60; // 4mhz Z80
  nDone[0]=nDone[1]=nDone[2]=nDone[3]=0;

  AftSoundFrameStart(); // start rendering this frames's sound

  SekOpen(0); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekRun(500); SekClose();
  SekOpen(1); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekRun(500); SekClose();
  SekOpen(2); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekClose();

  for (i=0;i<nInterleve;i++)
  {
    int j=0,nNext=0; // Next cycle count to run to
    int k=nInterleve-i-1;

    InputMemory();

    // Run all 68000s
    for (j=0;j<3;j++)
    {
      int bSleep;
      bSleep=CheckSleep(j);
      if (bSleep) continue; // See if this cpu can sleep
      SekOpen(j);
      nNext=(i+1)*nTotal[j]/nInterleve; // find out next cycle count to run to
      nDone[j]+=SekRun(nNext-nDone[j]);  // run cpu
      SekClose();
    }

    // Interleve the z80 a little bit (make sure we do it all though: hence 'k&15' not 'i&15' )
    j=3;
    if ((k&15)==0)
    {
      int nExe;
      nNext=(i+1)*nTotal[j]/nInterleve; // find out next cycle count to run to
      nExe=ZetRun(nNext-nDone[j]);   // run cpu
      nDone[j]+=nExe;
      nAftFmTimer-=nExe; // Move timer timeout back
      AftSoundRenderTo((i+1)*nBurnSoundLen/nInterleve); // Render sound for this portion
    }
  }

  AftSoundFrameStop(); // stop rendering this frames's sound

  if (pBurnDraw!=NULL) DrvDraw(); // Draw screen if needed
  return 0;
}

// Scan ram
int GalScan(int nAction,int *pnMin)
{
  struct BurnArea ba;

  if (nAction&4) // Scan volatile ram
  {
    if (pnMin!=NULL) *pnMin=0x009609; // Return minimum compatible version

    memset(&ba,0,sizeof(ba));
    ba.Data=RamStart;
    ba.nLen=RamEnd-RamStart;
    ba.szName="All Ram";
    BurnAcb(&ba);

    SekScan(nAction&3); // scan 68000 states

    CalcScan(); // Maths chip

    AftZScan(nAction&3); // Scan AB Z80
    PcmScan(); // Scan PCM chip
    SCAN_VAR(GalInput)
    SCAN_VAR(bLastGear)
  }

  return 0;
}
