#include "burnint.h"
// Sega PCM chip emulator

#define CHAN_COUNT (16) // must be a power of 2
#define FRAC_SHIFT (14) // accuracy of render

int nPcmRate=22050; // sample rate to render at
unsigned char *PcmRom=NULL; // Pointer to the Pcm banks
int nPcmBankCount=0; // Count of the number of banks
int nPcmBankSize=0; // Bank length is 1<<nPcmBankSize bytes
static int nPcmSpeed=15700;
static int nIncVal=0; // Amount to increment nChanPos each sample (* Reg[co+7])

static unsigned char Reg[0x100];
static int nChanPos[CHAN_COUNT]; // Position in the sample <<FRAC_SHIFT
static unsigned char nChanBank[CHAN_COUNT];

#define SET_START { nChanPos[c]=((Reg[co+5]<<8)|Reg[co+4])<<FRAC_SHIFT; }

int PcmInit()
{
  memset(Reg,0,sizeof(Reg));
  memset(&nChanPos,0,sizeof(nChanPos));
  memset(&nChanBank,0,sizeof(nChanBank));

  if (nPcmRate>0)
  {
    // Make the nChanPos increment value
    nIncVal=(nPcmSpeed<<FRAC_SHIFT)/nPcmRate;
  }
  return 0;
}

int PcmExit()
{
  nIncVal=0;
  return 0;
}

int PcmScan()
{
  SCAN_VAR(Reg)
  SCAN_VAR(nChanPos)
  SCAN_VAR(nChanBank)
  return 0;
}

int (*PcmGetBank) (int nReg86)=NULL;

// pcm update for one channel
static INLINE int ChannelUpdate(short *Dest,int nLen,int c)
{
  int co;
  int nStopAt=0;
  unsigned char *PcmBank=NULL;
  short *pd=NULL,*pEnd=NULL;
  c&=15; co=c<<3;

 // channel has reached the end/overflowed - stop or loop
#define CHECK_END \
  if (nChanPos[c] >= nStopAt) { if (Reg[co+0x86]&2) Reg[co+0x86]|=1; else SET_START  }

#define RET_IF_STOPPED \
  if (Reg[co+0x86]&1) { nChanPos[c]=0; return 0; }

  RET_IF_STOPPED // channel is stopped
  nStopAt=Reg[co+6]<<8; nStopAt&=(1<<nPcmBankSize)-1; // find stop position but make sure clipped
  nStopAt<<=FRAC_SHIFT; // convert to fractional value
  
  CHECK_END  RET_IF_STOPPED

  PcmBank=PcmRom+nChanBank[c]*(1<<nPcmBankSize);

  if (nLen<0) return 1;
  pd=Dest; pEnd=pd+(nLen<<1); // make pointers to the start and end of the area to render
  do
  {
    int p,sl,sr,l,r;
    p=nChanPos[c]>>FRAC_SHIFT; // Find the current sample position
    
    sl=PcmBank[p]-0x80; // Get the sample
    sl*=3; sl>>=1;
    sr=sl;
    
    sl*=Reg[co+2]&0x3f;
    sr*=Reg[co+3]&0x3f;

    // Add to the sound currently in the buffer
    l=pd[0]; r=pd[1];
    l+=sl; r+=sr;
#define CLIP(x) if (x<-0x8000) x=-0x8000;  if (x>0x7fff) x=0x7fff;
    CLIP(l) CLIP(r)
    pd[0]=(short)l; pd[1]=(short)r;
    pd+=2;

    nChanPos[c]+=nIncVal*Reg[co+7]/128; // increment sample position based on pitch
    CHECK_END  RET_IF_STOPPED // Quit if we reached the end of the sample
  }
  while (pd<pEnd);

  return 0;
}

int PcmUpdate(short *Dest,int nLen) // Add nLen*2 16-bit samples (stereo interleved) into Dest
{
  int i=0;
  if (PcmRom==NULL) return 1;

  for (i=0;i<CHAN_COUNT;i++) ChannelUpdate(Dest,nLen,i);
  return 0;
}

unsigned char PcmRead(unsigned short a)
{
  a&=0xff;
  return Reg[a];
}

void PcmWrite(unsigned short a,unsigned char d)
{
  int c=0; // Channel number 0-15
  int f=0; // function
  int co=0; // channel offset

  a&=0xff;

  // a=FCCC CFFF  channel number and function

  c=(a>>3)&15;
#if CHAN_COUNT<16
  c&=CHAN_COUNT-1;
#endif
  co=c<<3;
  f=a&0x87;

  // 86 &2 means don't loop
  // 86 &1 means stopped

  if (f==4 || f==5) SET_START
  if (f==0x86)
  {
    if ((Reg[a]&3)==0) ; // Looping already
    else
    {
      int b=0;
      // Find out the bank of this channel
      if (PcmGetBank!=NULL) b=PcmGetBank(d);
      
      if (b>=nPcmBankCount) b=0;
      nChanBank[c]=(unsigned char)b;

      SET_START
    }
  }

  Reg[a]=(unsigned char)d; // set register
}

