#include "burnint.h"
#include "cmc_int.h"
// Sega PCM chip emulator

#define CHAN_COUNT (16) // must be a power of 2
#define FRAC_SHIFT (12) // accuracy of render

int nPcmRate=22050; // sample rate to render at
unsigned char *PcmRom=NULL; // Pointer to the Pcm banks
int nPcmBankCount=0; // Count of the number of banks
int nPcmBankSize=0; // Bank length is 1<<nPcmBankSize bytes
static int nPcmSpeed=15700;
static int nIncVal=0; // Amount to increment nChanPos each sample (* Reg[co+7])

static unsigned char Reg[0x100];
static int nChanPos[CHAN_COUNT]; // Position in the sample <<FRAC_SHIFT
static unsigned char nChanBank[CHAN_COUNT];

#define SET_START { nChanPos[c]=((Reg[(c << 3)+5]<<8)|Reg[(c << 3)+4])<<FRAC_SHIFT; }

 // channel has reached the end/overflowed - stop or loop
#define CHECK_END \
  if (nChanPos[c] >= nStopAt) { if (Reg[(c << 3)+0x86]&2) Reg[(c << 3)+0x86]|=1; else SET_START  }

#define RET_IF_STOPPED \
  if (Reg[(c<<3)+0x86]&1) { nChanPos[c]=0; return 0; }



int PcmInit()
{
  memset(Reg,0,sizeof(Reg));
  memset(&nChanPos,0,sizeof(nChanPos));
  memset(&nChanBank,0,sizeof(nChanBank));

  if (nPcmRate>0)
  {
    // Make the nChanPos increment value
    nIncVal=(nPcmSpeed<<FRAC_SHIFT)/nPcmRate;
  }
  return 0;
}

int PcmExit()
{
  nIncVal=0;
  return 0;
}

int PcmScan()
{
  SCAN_VAR(Reg)
  SCAN_VAR(nChanPos)
  SCAN_VAR(nChanBank)
  return 0;
}

int (*PcmGetBank) (int nReg86)=NULL;


int PcmUpdate(short *Dest,int nLen) // Add nLen*2 16-bit samples (stereo interleved) into Dest
{
  int i=0;
  static int WorkSize=-1;
  static int *Work;
  int nStopAt=0;
  int Loop;

  if (PcmRom==NULL) return 1;
  if (nLen <= 0) { return 0; }

  if (!Precalc) { cmc_4p_Precalc(); }
  if (WorkSize < nLen)
  {
    if (Work) { free(Work); }
    WorkSize = nLen + (nLen&1);
    Work = (int *)malloc(sizeof(int)*2*WorkSize);
    WorkSize = nLen;
  }


  memset(Work, 0, nLen*2*sizeof(int));
  for (i=0;i<CHAN_COUNT;i++)// ChannelUpdate(Work,nLen,i);
  {
    if (!(Reg[(i<<3)+0x86]&1))
    {
      nStopAt=Reg[(i << 3)+6]<<8;
      nStopAt&=(1<<nPcmBankSize)-1; // find stop position but make sure clipped
      nStopAt<<=FRAC_SHIFT; // convert to fractional value
  
      if (!(Reg[(i << 3)+0x86]&2))
      {
        Loop = nStopAt-(((Reg[(i << 3)+5]<<8)|Reg[(i << 3)+4])<<FRAC_SHIFT);
      }
      else { Loop = 0; }

      if (!ChannelMix_8U(Work, nLen,
		      PcmRom+nChanBank[i]*(1<<nPcmBankSize),
		      nStopAt,
		      nChanPos + i,
  	              (Reg[(i << 3)+2]&0x3f)*3/2,
		      (Reg[(i << 3)+3]&0x3f)*3/2,
	              Loop,
	              nIncVal*Reg[(i << 3)+7]/128) )
      {
        Reg[(i<<3)+0x86] |= 1;
      }
    }
  }

  AdaptSoundBuff_Add(Work, Dest, nLen);

  return 0;
}

unsigned char PcmRead(unsigned short a)
{
  a&=0xff;
  return Reg[a];
}

void PcmWrite(unsigned short a,unsigned char d)
{
  int c=0; // Channel number 0-15
  int f=0; // function
  int co=0; // channel offset

  a&=0xff;

  // a=FCCC CFFF  channel number and function

  c=(a>>3)&15;
#if CHAN_COUNT<16
  c&=CHAN_COUNT-1;
#endif
  co=c<<3;
  f=a&0x87;

  // 86 &2 means don't loop
  // 86 &1 means stopped

  if (f==4 || f==5) SET_START
  if (f==0x86)
  {
    if ((Reg[a]&3)==0) ; // Looping already
    else
    {
      int b=0;
      // Find out the bank of this channel
      if (PcmGetBank!=NULL) b=PcmGetBank(d);
      
      if (b>=nPcmBankCount) b=0;
      nChanBank[c]=(unsigned char)b;

      SET_START
    }
  }

  Reg[a]=(unsigned char)d; // set register
}

